# -*- Mode: Perl -*-

#
# Copyright (c) 2001, Bryan Jurish.  All rights reserved.
#
# This package is free software.  You may redistribute it
# and/or modify it under the same terms as Perl itself.
#

###############################################################
#
#      File: Data::FStructure::List.pm
#    Author: Bryan Jurish <jurish@ling.uni-potsdam.de>
#
# Description: Feature-structures for perl: linked lists
#
###############################################################

package Data::FStructure::List;
# System modules
# 3rd party exstensions
# user extension modules
use Data::FStructure::Std;
@ISA       = qw(Data::FStructure::Std Exporter);
@EXPORT    = qw();
@EXPORT_OK = (
	      qw($LIST_FTR_FIRST $LIST_FTR_NEXT),
	     );
%EXPORT_TAGS =
  (
   'ftrvars' => [ qw($LIST_FTR_FIRST $LIST_FTR_NEXT) ],
  );

###############################################################
# Package-global variables
###############################################################
our $VERSION = 0.03;

# list-features, lisp-style
#our ($LIST_FTR_FIRST,$LIST_FTR_NEXT) = qw(car cdr);

# list-features, prolog-/Carpenter-style
#our ($LIST_FTR_FIRST,$LIST_FTR_NEXT) = qw(head tail);

# list-features, mnemonic
our ($LIST_FTR_FIRST,$LIST_FTR_NEXT) = qw(first next);


# a handy alias
*LIST_FTR_REST = \$LIST_FTR_NEXT;


###############################################################
# Initialization:
# $fs->initialize(\%args)
# + new keys:
#     list_ftr_first => $str,
#     list_ftr_next => $regex,
###############################################################
sub initialize {
  my ($fs,$args) = @_;
  $fs->{list_ftr_first} = $args->{list_ftr_first} if exists($args->{list_ftr_first});
  $fs->{list_ftr_next} = $args->{list_ftr_next} if exists($args->{list_ftr_next});
  return $fs;
}


###############################################################
# sneakily switch packages
###############################################################
package Data::FStructure::Base;
Data::FStructure::List->import qw(:ftrvars);

## set perldata vars if available -- not by default
#if (defined(%Data::FStructure::perldata::SPIT_TYPE2REF)
#    && defined(%Data::FStructure::perldata::SPIT_LINKUS)) {
#  $Data::FStructure::perldata::SPIT_TYPE2REF{list} =
#    \&_type2ref_list;
#  $Data::FStructure::perldata::SPIT_LINKUS{list} =
#    \&_linkus_list;
#}

###############################################################
# Accessors/manipulators
###############################################################

# $ftr = $fs->list_ftr_first();
# $fs->list_ftr_first($ftr);
*list_ftr_car = \&list_ftr_first;
*list_ftr_head = \&list_ftr_first;
sub list_ftr_first {
  return
    (exists($_[1])
     ? (ref($_[0])
	? ($_[0]->{list_ftr_first} = $_[1])
	: ($LIST_FTR_FIRST = $_[1]))
     : (ref($_[0])
	? (defined($_[0]->{list_ftr_first})
	   ? $_[0]->{list_ftr_first}
	   : $LIST_FTR_FIRST)
	: $LIST_FTR_FIRST));
}

# $ftr = $fs->list_ftr_next();
# $fs->list_ftr_next($ftr);
*list_ftr_cdr = \&list_ftr_next;
*list_ftr_rest = \&list_ftr_next;
*list_ftr_tail = \&list_ftr_next;
sub list_ftr_next {
  return
    (exists($_[1])
     ? (ref($_[0])
	? ($_[0]->{list_ftr_next} = $_[1])
	: ($LIST_FTR_NEXT = $_[1]))
     : (ref($_[0])
	? (defined($_[0]->{list_ftr_next})
	   ? $_[0]->{list_ftr_next}
	   : $LIST_FTR_NEXT)
	: $LIST_FTR_NEXT));
}

# $fs->list_ftr_style('prolog'|'lisp'|'explicit');
sub list_ftr_style {
  if ($_[1] =~ /^lisp$/i) {
    $_[0]->list_ftr_first('car');
    $_[0]->list_ftr_next('cdr');
  } elsif ($_[1] =~ /^prolog$/i) {
    $_[0]->list_ftr_first('head');
    $_[0]->list_ftr_next('tail');
  } else {
    # defaults
    $_[0]->list_ftr_first('first');
    $_[0]->list_ftr_next('cdr');
  }
}


# $bool = $fs->is_empty_list;
# ALIASES: is_elist
*is_elist = \&is_empty_list;
sub is_empty_list { return $_[0]->is_empty; }

# $first_node = $fs->first();
# $id = $fs->first($node)
# ----> retains root
*car_node = \&first_node;
*head_node = \&first_node;
sub first_node {
  return undef unless (defined($_[0]->root));
  if (exists($_[1])) {
    my $node = $_[0]->add_node($_[1]);
    $_[0]->delta($_[0]->root,$_[0]->list_ftr_first,$node);
    return $id;
  }
  return $_[0]->delta($_[0]->root,$_[0]->list_ftr_first);
}

# $rest_node = $fs->rest();
# ----> retains root
*cdr_node = \&next_node;
*tail_node = \&next_node;
*rest_node = \&next_node;
sub next_node {
  return undef unless (defined($_[0]->root));
  if (exists($_[1])) {
    my $node = $_[0]->add_node($_[1]);
    $_[0]->delta($_[0]->root,$_[0]->list_ftr_next,$node);
    return $id;
  }
  return $_[0]->delta($_[0]->root,$_[0]->list_ftr_next);
}

# $eltnode = $fs->pop_node();
# ---> alters root!
sub pop_node {
  return undef unless (defined($_[0]->root));
  my $popped = $_[0]->first_node;
  $_[0]->root($_[0]->rest_node);
  return $popped;
}

# $newroot = $fs->push_node($node);
# ---> alters root!
sub push_node {
  my ($cons,$pushme) = $_[0]->add_nodes(undef,$_[1]);
  $_[0]->delta($cons,$_[0]->list_ftr_next,$_[0]->root);
  $_[0]->delta($cons,$_[0]->list_ftr_first,$pushme);
  $_[0]->root($cons);
  return $cons;
}

# $nodeid = $fs->nth_node($i)
sub nth_node {
  my ($fs,$i) = @_;
  return undef unless ($i >= 0);
  my $node = $fs->root;
  for ( ; $i > 0; --$i) {
    return undef unless (defined($node = $fs->delta($node,$_[0]->list_ftr_next)));
  }
  return $fs->delta($node,$_[0]->list_ftr_first);
}



# $fs2 = $fs1->pop_elt()
sub first_elt { return $_[0]->substructure($_[0]->first_node); }

# $fs2 = $fs1->pop_elt()
sub pop_elt { return $_[0]->substructure($_[0]->pop_node); }

# $newelt = $fs->push_elt($val);
sub push_elt {
  my ($cons) = $_[0]->add_node(undef);
  if (defined($_[0]->root)) {
    $_[0]->delta($cons,$_[0]->list_ftr_next,$_[0]->root);
  }
  $_[0]->root($cons);
  return $_[0]->_set_pvalue([$_[0]->list_ftr_first],$_[1]);
}


# $fs2 = $fs1->nth_elt($i);
*nth = \&nth_elt;
sub nth_elt {
  return $_[0]->substructure($_[0]->nth_node($_[1]));
}


# $length = $fs->list_length()
# ---> returns undef on cyclic lists, 0 on non-lists
sub list_length {
  my $fs = shift;
  my $node = $fs->root;
  my %visited = ();
  my ($len);
  for ($len = 0; defined($node); ++$len) {
    return undef if (exists($visited{$node}));
    $visited{$node} = '';
    $node = $fs->delta($node,$fs->list_ftr_next);
  }
  return $len;
}

# $ref = type2ref_callback($fs,$type)
sub _type2ref_list { return defined($_[1]) ? bless([],$_[1]) : []; }

# undef = &linkus_callback($fs,$qfrom,$f,$qto,\%node2ref);
sub _linkus_list {
  my ($fs,$qfrom,$f,$qto,$node2ref) = @_;
  if ($f eq $fs->list_ftr_first) {
    push(@{$node2ref->{$qfrom}},$node2ref->{$qto});
  }
  elsif ($f eq $fs->list_ftr_next) {
    unless ($node2ref->{$qfrom} eq $node2ref->{$qto}) {
      # ignore cyclic lists
      push(@{$node2ref->{$qfrom}}, @{$node2ref->{$qto}});
      $node2ref->{$qto} = $node2ref->{$qfrom};
    }
  } else {
    carp("Warning: non-list feature in _linkus_list() ignored!");
  }
}

# \@listref = $fs->as_listref
# ---> list of substructures
sub as_listref {
  my $fs = shift;
  my $node = $fs->root;
  my %visited = ();
  my %evisited = ();
  my @array = qw();
  my ($i,$nodei,$type);
  for ($i = 0; defined($node); ++$i) {
    return undef if (exists($visited{$node})); # no cyclic lists
    $visited{$node} = '';
    if (defined($nodei = $fs->delta($node,$fs->list_ftr_first))) {
      $array[$i] =
	exists($evisited{$nodei})
	  ? $array[$evisited{$nodei}]
	  : $fs->substructure($nodei);
      $evisited{$nodei} = $i;
    }
    $node = $fs->delta($node,$fs->list_ftr_next);
  }
  return \@array;
}

# $fs->_gobble_array($arrayref,\%vals2nodes,\@stack,\%opts)
sub _gobble_list {
  my ($fs,$any,$any2node,$stack) = @_;

  my ($cons,$next);
  push(@$stack,
       map {
	 $next = $cons;
	 $fs->add_node($cons = $fs->_gensym_node);
	 $fs->delta($cons,$fs->list_ftr_next,$next);
	 ($cons,$fs->list_ftr_first,$_)
       } reverse(@$any));

  # need new node if _gobble_array() was called...
  $any2node->{$any} = $cons;

  # set up type assignment for (blessed) references
  $fs->theta($cons, ref($any)) if (ref($any) ne 'ARRAY');
}


1;
__END__

# -*- Mode: Perl -*-
=pod

=head1 NAME

Data::FStructure::List - Feature structures for perl -- linked lists.

=head1 SYNOPSIS

  use Data::FStructure::List;

  #
  # Package Variables
  #
  $LIST_FTR_FIRST = 'first';         # points to 'current' element (default)
  $LIST_FTR_NEXT = 'next';           # points to next list-cell (default)

  #
  # Construction
  #
  $listfs = Data::FStructure::Base->new(\%args);   # new list
  Data::FStructure::List->initialize(\%args);      # intialization

  #
  # Feature Naming Conventions
  #
  $fs->list_ftr_first();                         # get 'first' feature-name
  $fs->list_ftr_first($ftr);                     # set 'first' name for object
  Data::FStructure::Base->list_ftr_first($ftr);   # ... or class-wide default

  $fs->list_ftr_next();                          # get 'next' feature-name
  $fs->list_ftr_next($ftr);                      # set 'next' name for object
  Data::FStructure::Base->list_ftr_next($ftr);    # ... or class-wide default

  $fs->list_ftr_style($style);                   # 'lisp'|'prolog'|'default'
  Data::FStructure::Base->list_ftr_style($style); # ... set package defaults

  #
  # List Access
  #
  $bool = $fs->is_empty_list();       # alias for 'is_empty()'
  $n_or_top = $fs->list_length();     # get list length

  #
  # Substructure-Level List Access
  #
  $fs2 = $fs->first_elt();            # get first substructure
  $fs2 = $fs->pop_elt();              # get first substructure & re-root
  $fs2 = $fs->push_elt($elt);         # set first substructure & re-root

  $fs2 = $fs1->nth_elt($elt);         # set substructure by numeric index

  #
  # Node-Level List Access
  #
  $node = $fs->first_node();          # get current element
  $fs->first_node($node);             # set current element

  $node = $fs->next_node();           # get next list-cell
  $fs->next_node($node);              # set next list-cell

  $node = $fs->pop_node();            # get first node & re-root
  $root = $fs->push_node($node);      # re-root & set first node

  $node = $fs->nth_node($n);          # node-lookup by numeric index

  #
  # perldata Interface
  #
  $listref = $fs->as_listref();                    # convert to perl array

  $fs->_gobble_list(\@ary,\%v2n,\@stack,\%opts);   # for gobble()
  $ref = _type2ref_list($fs,$type);                # for spit()
  &linkus_callback($fs,$qfrom,$f,$qto,\%node2ref); # for spit()


=head1 REQUIREMENTS

Carp,

=cut

#--------------------------------------------------------------
# Description
#--------------------------------------------------------------
=pod

=head1 DESCRIPTION


=cut

#--------------------------------------------------------------
# Variables
#--------------------------------------------------------------
=pod

=head1 PACKAGE VARIABLES

C<Data::FStructure::List> defines the following variables,
and imports them into C<Data::FStructure::Base>.

=over 4

=item * C<$LIST_FTR_FIRST>

The name of the feature by which the element
associated with a cell of a linked list may be
accessed.  The default value is C<'first'>.

See Also: C<list_ftr_style()>.

=item * C<$LIST_FTR_NEXT>

The name of the feature by which the next cell
of a linked list may be accessed.
The default value is C<'next'>.

See Also: C<list_ftr_style()>.

=back

=cut

#--------------------------------------------------------------
# Methods
#--------------------------------------------------------------
=pod

=head1 METHODS

=cut

#--------------------------------------------------------------
# Construction
#--------------------------------------------------------------
=pod

=head2 Construction & Initialization

=over 4

=item * C<new(\%args)>

Just like the C<Data::FStructure::Std> method C<new()>, but
additionally recognizes the following keys
of C<\%args>:

=over 4

=item * C<< list_ftr_first =E<gt> $ftr >>

Sets the name of the "first" feature for the new
instance to C<$ftr>.

=item * C<< list_ftr_next =E<gt> $ftr >>

Sets the name of the "first" feature for the new
instance to C<$ftr>.

=back

=back

=cut


#--------------------------------------------------------------
# Feature-Naming Conventions
#--------------------------------------------------------------
=pod

=head2 Feature-Naming Conventions

The linked-list implementation in C<Data::FStructure::List>
relies on two feature-names used in the representation of
list cells.  The names of these features can be looked up
or set with the methods described below, which may be called
either as instance- or as class-methods (they are added to
C<Data::FStructure::Base>).

If called
as class methods, they get/set the package variables
C<$LIST_FTR_FIRST> and C<$LIST_FTR_NEXT>; if called
as instance methods, they get/set instance-specific values,
using the class-wide defaults as a fallback.

=over 4

=item * C<list_ftr_first()>, C<list_ftr_first($ftr)>

Get/set the name of the feature pointing from a list-cell
to its first element.

=item * C<list_ftr_next()>, C<list_ftr_next($ftr)>

Get/set the name of the feature pointing from a list-cell
to the next cell in the list.

=item * C<list_ftr_style($style)>

Get/set both feature-names in one call.

If C<$style> is C<'lisp'>, then the "first"
feature is set to C<'car'> and the "next" feature
is set to C<'cdr'>.

Otherwise, if C<$style> is C<'prolog'>, then
the "first" feature
is set to C<'head'> and the "next" feature is set
to C<'tail'>.

Otherwise, the "first" feature is set to
C<'first'>, and the "next" feature is set to C<'next'>
(which are the default values, anyway).

=back

=cut


#--------------------------------------------------------------
# List Access
#--------------------------------------------------------------
=pod

=head2 List Access

C<Data::FStructure::List> defines the follwing methods
in C<Data::FStructure::Base>,
which may be used to gather information about
list structures.

=over 4

=item * C<is_empty_list()>

Really just an alias for C<Data::FStructure::Base::is_empty()>.

=item * C<list_length()>

Returns the length of the list a numeric value, or
C<$TYPE_TOP> if the list is cyclic with respect to
the "next" feature.

=back

=cut


#--------------------------------------------------------------
# Footer
#--------------------------------------------------------------
=pod

=head2 Substructure-Level List Access

C<Data::FStructure::List> defines the follwing methods
in C<Data::FStructure::Base>,
which may be used for accessing list-structures at
the substructure-level.


=over 4

=item * C<first_elt()>

Returns the substructure whose root is reached by following
the "first" feature from the root of the calling instance, or
C<undef> if there is no such feature.

=item * C<pop_elt()>

Like C<first_elt()>, but re-roots the calling instance to
the node reached by following the "next" feature from
its current root.

B<Note>: this method does B<NOT> alter any transition values
in embedding superstructures -- if you need to do this,
use C<pvalue()> or C<gobble()>.


=item * C<push_elt($elt)>

Creates a new list cell to hold C<$elt> and re-roots the
FStructure to this newly created cell.  Sets the value
of the new cell's "next" feature to the old root root
node, if any.  Sets the value of the new cell's "first"
feature to C<$elt> using the C<pvalue()> method.
Returns the FStructure object.

Same caveats as for C<pop_elt()>.

=item * C<nth_elt($i)>

Returns the C<$i>th element of the the FStructure as a
substructure, or C<undef> if there is no such element.

=back

=cut


#--------------------------------------------------------------
# Footer
#--------------------------------------------------------------
=pod

=head2 Node-Level List Access

C<Data::FStructure::List> defines the follwing methods
in C<Data::FStructure::Base>,
which may be used for accessing list-structures at
the node-level.

=over 4

=item * C<first_node()>, C<first_node($node)>

Like C<first_elt()>, but gets/sets/returns a node-name.

=item * C<next_node()>

Like C<next_elt()>, but gets/sets/returns a node-name.

=item * C<pop_node()>

Like C<pop_elt()>, but gets/sets/returns a node-name.

=item * C<push_node($node)>

Like C<push_elt()>, but gets/sets/returns a node-name.

=item * C<nth_node($n)>

Like C<nth_elt()>, but gets/sets/returns a node-name.

=back

=cut


#--------------------------------------------------------------
# Footer
#--------------------------------------------------------------
=pod

=head2 perldata Interface

C<Data::FStructure::List> defines the follwing methods
in C<Data::FStructure::Base>,
which may be used by the C<Data::FStructure::perldata> interface.

=over 4

=item * C<as_listref()>

Returns a reference to an array of substructures, each
representing an element of the list, or C<undef> if
the list is cyclic with respect to the "next" feature.

=item * C<_gobble_list()>

Interface to the C<Data::FStructure::perldata::gobble()> method
to read perl list-references into an FStructure object
as linked lists.

=item * C<_type2ref_list($fs,$type)>

Interface to the C<Data::FStructure::perldata::spit()> method
to construct perl array-references from an FStructure
linked list.

If the C<Data::FStructure::perldata> module is used,
this method may be associated with
this method in C<%Data::FStructure:::perldata::SPIT_TYPE2REF>,
although this is not done by default.

=item * C<_linkus_list($fs,$qfrom,$f,$qto,\%node2ref)>

Interface to the C<Data::FStructure::perldata::spit()> method
to construct perl array-references from an FStructure
linked list.  May produce odd results on cyclic lists.

If the C<Data::FStructure::perldata> module is used,
this method may be associated with
this method in C<%Data::FStructure:::perldata::SPIT_LINKUS>,
although this is not done by default.

=back

=cut


#--------------------------------------------------------------
# Footer
#--------------------------------------------------------------
=pod

=head1 NOTES

The C<Data::FStructure::List> module does not define any special type
or node-name for the "empty list".  C<undef> values
(also returned for missing transitions) are interpreted
as end-of-list.

=head1 BUGS

There should be a method C<nth_elt($i,$elt)> which sets
the C<n>th element, but there isn't.

=head1 ACKNOWLEDGEMENTS

perl by Larry Wall.

=head1 AUTHOR

Bryan Jurish E<lt>jurish@ling.uni-potsdam.deE<gt>

=head1 COPYRIGHT

Copyright (c) 2001, Bryan Jurish.  All rights reserved.

This package is free software.  You may redistribute it
and/or modify it under the same terms as Perl itself.

=head1 SEE ALSO

Bob Carpenter, I<The Logic of Typed Feature Structures>.
Cambridge University Press, 1992.

perl(1).
Data::FStructure(3pm).
Data::FStructure::Std(3pm).
Data::FStructure::perldata(3pm).
