# -*- Mode: Perl -*-
# t/08_appr.t : test Approp

$TEST_DIR = './t';
#use lib qw(../blib/lib); $TEST_DIR = '.'; # for debugging

# load what we'll need
use Data::FStructure;
use Data::FStructure::pretty;
use Data::FStructure::Approp;
do "$TEST_DIR/common.plt";

plan(test => 18);

# prep
$h = attitudes_hi();
$fs = Data::FStructure->new({hierarchy=>$h});
$fs->from_text('[ (value) ARG: [ (true) ] ]');
# 1-2: inappropriate fs: the type `value' takes no features
isok('is_wt [no approp ftrs]',!$fs->is_wt());
isok('is_twt [no approp ftrs]',!$fs->is_twt());

# 3-4: just plain wrong fs: 'denial' takes 'false' as a value
$fs->from_text('[(denial) ARG: [ (true) ]]');
isok('is_wt [wrong value]',!$fs->is_wt());
isok('is_twt [wrong value]',!$fs->is_twt());

# 5-6: wt=1, twt=0 -- missing ftr 'ARG'
$fs->from_text('[ (curiosity) ]');
isok('is_wt [missing ftr]',$fs->is_wt());
isok('is_twt [missing ftr]',!$fs->is_twt());

# 7-8: everything's kosher
$fs->from_text('[(skepsis) ARG: [(maybenot) ]]');
isok('is_wt [kosher - direct]',$fs->is_wt());
isok('is_twt [kosher - direct]',$fs->is_twt());


# 9-10: wt=0,twt=0: inherited specificity: 'curiosity' should take 'whoknows'
$fs->from_text('[(curiosity) ARG: [(maybe) ]]');
isok('is_wt [overgeneral - inherited]',!$fs->is_wt());
isok('is_twt [overgeneral - inherited]',!$fs->is_twt());

# 11-12: wt=1, twt=1 -- should just do subsumption-tests
$fs->from_text('[(curiosity) ARG: [(whoknows) ]]');
isok('is_wt [kosher - inherited]',$fs->is_wt());
isok('is_twt [kosher - inherited]',$fs->is_twt());

# 13-14: wt=1, twt=1 -- non-deterministic inheritance
$fs->from_text('[(contradiction) ARG: [(maybenot) ]]');
isok('is_wt [nondet (kinda kosher) - inherited]',$fs->is_wt());
isok('is_twt [nondet (kinda kosher) - inherited]',$fs->is_twt());

# 15-16: nondet-warning
$thewarning = '';
$SIG{__WARN__} = sub { $thewarning = $_[0]; };
$oldW = $^W;
$^W = 1;
$Math::PartialOrder::VERBOSE = 1;
$appr = $h->get_approp_det(qw(contradiction ARG));
$^W = $oldW;
delete($SIG{__WARN__});
isok('get_approp_det [nondet - result]', $appr =~ /^maybe(not)?/);
isok('get_approp_det [nondet - warning]',
     $thewarning =~ /^Warning: unsupported deterministic operation in non-ccpo hierarchy/);

# 17-18: get approp-hash
isok('get_approp_det [lookup]',($appr = $h->get_approp_det('curiosity')));
hashok('get_approp_det [result]',$appr, {ARG=>'whoknows'});


print "\n";
# end of t/08_appr.t

